#include <pebble.h>

static const SmartstrapServiceId SERVICE_ID = 0x1001;
static const SmartstrapAttributeId LAMP_ATTRIBUTE_ID = 0x0003;
static const size_t LAMP_ATTRIBUTE_LENGTH = 2;

static SmartstrapAttribute *led_attribute;
static SmartstrapAttribute *uptime_attribute;
static SmartstrapAttribute *lamp_attribute;

static Window *window = NULL;
static TextLayer *status_text_layer = NULL;
static TextLayer *uptime_text_layer = NULL;

typedef struct __attribute__((packed)){
  uint8_t channel;
  uint8_t value;
} DMXP;

static void prv_availability_changed(SmartstrapServiceId service_id, bool available) {
  if (service_id != SERVICE_ID) {
    return;
  }

  if (available) {
    text_layer_set_background_color(status_text_layer, GColorGreen);
    text_layer_set_text(status_text_layer, "Connected!");
      text_layer_set_text(uptime_text_layer, "Ready");
  } else {
    text_layer_set_background_color(status_text_layer, GColorRed);
    text_layer_set_text(status_text_layer, "Disonnected!");
      text_layer_set_text(uptime_text_layer, "X");
  }
}

static void prv_set_led_attribute(int chan, int val) {
  SmartstrapResult result;
  uint8_t *buffer;
  size_t length;
  result = smartstrap_attribute_begin_write(lamp_attribute, &buffer, &length);
  if (result != SmartstrapResultOk) {
    //APP_LOG(//APP_LOG_LEVEL_ERROR, "Begin write failed with error %d", result);
    return;
  }

  DMXP cmd = {.channel = chan, .value = val};

  memcpy(buffer,&cmd,sizeof(cmd));

  result = smartstrap_attribute_end_write(lamp_attribute, sizeof(cmd), false);
  if (result != SmartstrapResultOk) {
    //APP_LOG(//APP_LOG_LEVEL_ERROR, "End write failed with error %d", result);
    return;
  }
}

static void up_click_handler(ClickRecognizerRef recognizer, void *context) {
  prv_set_led_attribute(1,255);
  prv_set_led_attribute(2,255);
  prv_set_led_attribute(3,255);
  prv_set_led_attribute(4,255);
  prv_set_led_attribute(5,255);
  prv_set_led_attribute(6,255);

}

static void down_click_handler(ClickRecognizerRef recognizer, void *context) {
  prv_set_led_attribute(1,0);
  prv_set_led_attribute(2,0);
  prv_set_led_attribute(3,0);
  prv_set_led_attribute(4,0);
  prv_set_led_attribute(5,0);
  prv_set_led_attribute(6,0);
  
}

static void click_config_provider(void *context) {
  window_single_click_subscribe(BUTTON_ID_UP, up_click_handler);
  window_single_click_subscribe(BUTTON_ID_DOWN, down_click_handler);
}

static void window_load(Window *window) {
  window_set_background_color(window, GColorWhite);

  status_text_layer = text_layer_create(GRect(0, 0, 144, 40));
  text_layer_set_font(status_text_layer, fonts_get_system_font(FONT_KEY_GOTHIC_28));
  text_layer_set_text_alignment(status_text_layer, GTextAlignmentCenter);
  text_layer_set_text(status_text_layer, "");
  layer_add_child(window_get_root_layer(window), text_layer_get_layer(status_text_layer));
  
  uptime_text_layer = text_layer_create(GRect(0, 60, 144, 40));
  text_layer_set_font(uptime_text_layer, fonts_get_system_font(FONT_KEY_GOTHIC_28));
  text_layer_set_text_alignment(uptime_text_layer, GTextAlignmentCenter);
  text_layer_set_text(uptime_text_layer, "X");
  layer_add_child(window_get_root_layer(window), text_layer_get_layer(uptime_text_layer));

  prv_availability_changed(SERVICE_ID, smartstrap_service_is_available(SERVICE_ID));

}

static void window_unload(Window *window) {
  text_layer_destroy(status_text_layer);
  text_layer_destroy(uptime_text_layer);
}

static void own_handler(DictionaryIterator *iter, void *context){
    Tuple *t = dict_read_first(iter);
    while(t != NULL){
      prv_set_led_attribute(t->key,t->value->int32);
      t = dict_read_next(iter);
    }
}

static void init(void) {
  // setup window
  //APP_LOG(APP_LOG_LEVEL_ERROR,"Opened");
  window = window_create();
  window_set_click_config_provider(window, click_config_provider);
  window_set_window_handlers(window, (WindowHandlers) {
    .load = window_load,
    .unload = window_unload,
  });
  const bool animated = true;
  window_stack_push(window, animated);
  // setup smartstrap
  SmartstrapHandlers handlers = (SmartstrapHandlers) {.availability_did_change = prv_availability_changed};
  smartstrap_subscribe(handlers);
  lamp_attribute = smartstrap_attribute_create(SERVICE_ID, LAMP_ATTRIBUTE_ID, LAMP_ATTRIBUTE_LENGTH);

  app_message_register_inbox_received(own_handler);
  app_message_open(app_message_inbox_size_maximum(), app_message_outbox_size_maximum());


}


static void deinit(void) {
  window_destroy(window);
  smartstrap_attribute_destroy(lamp_attribute);
}

int main(void) {
  init();
  app_event_loop();
  deinit();
}
